<?php
/*
Plugin Name: User Locker
Plugin URI: http://www.poradnik-webmastera.com/projekty/user_locker/
Description: This plugin locks user account after given number of incorrect login attempts.
Author: Daniel Frużyński
Version: 1.1.10
Author URI: http://www.poradnik-webmastera.com/
Text Domain: user-locker
*/

/*  Copyright 2009-2010  Daniel Frużyński  (email : daniel [A-T] poradnik-webmastera.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


if ( !class_exists( 'UserLocker' ) ) {

class UserLocker {
	var $pass_check = false;
	var $locked = false;
	
	// Constructor
	function UserLocker() {
		// Initialise plugin
		add_action( 'init', array( &$this, 'init' ) );
		add_action( 'admin_init', array( &$this, 'admin_init' ) );
		
		// Add option to Admin menu
		add_action( 'admin_menu', array( &$this, 'admin_menu' ) );
		
		// Check if user is already locked
		add_filter( 'wp_authenticate_user', array( &$this, 'wp_authenticate_user' ), 1 );
		
		// Set password check flag
		add_filter( 'check_password', array( &$this, 'check_password' ) );

		// Increment bad attempt counter and finally lock account
		add_action( 'wp_login_failed', array( &$this, 'wp_login_failed' ) );
		
		// Reset account lock on pass reset and valid login
		add_action( 'password_reset', array( &$this, 'password_reset' ) );
		add_action( 'wp_login', array( &$this, 'wp_login' ) );
		
		// Add info about account lock
		add_filter( 'login_errors', array( &$this, 'login_errors' ) );
		
		// Edit user profile
		add_action( 'edit_user_profile', array( &$this, 'edit_user_profile' ) );
		add_action( 'edit_user_profile_update', array( &$this, 'edit_user_profile_update' ) );
		
		// Add new column to the user list
		add_filter( 'manage_users_columns', array( &$this, 'manage_users_columns' ) );
		add_filter( 'manage_users_custom_column', array( &$this, 'manage_users_custom_column' ), 10, 3 );
	}
	
	// Initialise plugin
	function init() {
		load_plugin_textdomain( 'user-locker', false, dirname( plugin_basename( __FILE__ ) ).'/lang' );
	}
	
	// Initialise plugin - admin part
	function admin_init() {
		register_setting( 'user-locker', 'userlocker_max_attempts', array( &$this, 'sanitize_nonnegative' ) );
	}
	
	// Add option to Admin menu
	function admin_menu() {
		add_submenu_page( 'options-general.php', 'User Locker', 
			'User Locker', 'manage_options', __FILE__, array( &$this, 'options_panel' ) );
	}
	
	// Check if user is already locked
	function wp_authenticate_user( $user ) {
		if ( is_wp_error( $user ) ) {
			return $user;
		}
		
		$disabled = get_user_option( 'ul_disabled', $user->ID, false );
		if ( $disabled ) {
			return new WP_Error('ul_user_disabled', __('<strong>ERROR</strong>: This user account is disabled.', 'user-locker'));
		}
		
		$locked = get_user_option( 'ul_locked', $user->ID, false );
		if ( $locked ) {
			return new WP_Error('ul_user_locked', __('<strong>ERROR</strong>: This user account is locked for security reasons. Please use Lost Password option to unlock it.', 'user-locker'));
		}
		
		return $user;
	}
	
	// Set password check flag
	function check_password( $check ) {
		$this->pass_check = true;
		return $check;
	}
	
	// Increment bad attempt counter and finally lock account
	function wp_login_failed( $username ) {
		if ( !$this->pass_check ) {
			// Function called too early
			return;
		}
		
		$user = get_userdatabylogin($username);
		if ( !$user || ($user->user_login != $username) ) {
			// Invalid username
			return;
		}
		
		$disabled = get_user_option( 'ul_disabled', $user->ID, false );
		$locked = get_user_option( 'ul_locked', $user->ID, false );
		if ( !$disabled && !$locked ) {
			$cnt = get_user_option( 'ul_bad_attempts', $user->ID, false );
			if ( $cnt === false ) {
				$cnt = 1;
			} else {
				++$cnt;
			}
			update_user_option( $user->ID, 'ul_bad_attempts', $cnt, false );
			
			if ( $cnt >= get_option( 'userlocker_max_attempts' ) ) {
				$this->locked = true;
				update_user_option( $user->ID, 'ul_locked', true, false );
			}
		}
	}
	
	// Reset account lock on pass reset
	function password_reset( $user ) {
		$this->unlock_user( $user->ID );
	}
	
	// Reset account lock on valid login
	function wp_login( $username ) {
		$user = get_userdatabylogin( $username );
		$this->unlock_user( $user->ID );
	}
	
	// Unlock account for given user
	function unlock_user( $user_id ) {
		update_user_option( $user_id, 'ul_bad_attempts', 0, false );
		update_user_option( $user_id, 'ul_locked', false, false );
	}
	
	// Add info about account lock
	function login_errors( $errors ) {
		if ( $this->locked ) {
			$errors .= __('<strong>ERROR</strong>: This user account has been locked for security reasons. Please use Lost Password option to unlock it.', 'user-locker') . "<br />\n";
		}
		return $errors;
	}
	
	function edit_user_profile() {
		if ( !current_user_can( 'edit_users' ) ) {
			return;
		}
		
		global $user_id;
		
		// User cannot disable itself
		$current_user = wp_get_current_user();
		$current_user_id = $current_user->ID;
		if ( $current_user_id == $user_id ) {
			return;
		}
?>
<h3><?php _e('User Locking', 'user-locker') ?></h3>
<table class="form-table">
<tr>
	<th scope="row"><?php _e('User account locked', 'user-locker'); ?></th>
	<td><label for="ul_locked"><input name="ul_locked" type="checkbox" id="ul_locked" value="false" <?php checked(true, get_user_option( 'ul_locked', $user_id, false )); ?> /> <?php _e('User account is locked for security reasons', 'user-locker'); ?></label></td>
</tr>
<tr>
	<th scope="row"><?php _e('User account disabled', 'user-locker'); ?></th>
	<td><label for="ul_disabled"><input name="ul_disabled" type="checkbox" id="ul_disabled" value="false" <?php checked(true, get_user_option( 'ul_disabled', $user_id, false )); ?> /> <?php _e('User account is disabled', 'user-locker'); ?></label></td>
</tr>
</table>
<?php
	}
	
	function edit_user_profile_update() {
		if ( !current_user_can( 'edit_users' ) ) {
			return;
		}
		
		global $user_id;
		
		// User cannot disable itself
		$current_user = wp_get_current_user();
		$current_user_id = $current_user->ID;
		if ( $current_user_id == $user_id ) {
			return;
		}
		
		$locked = get_user_option( 'ul_locked', $user_id, false );
		$new_locked = isset( $_POST['ul_locked'] );
		
		if ( $locked != $new_locked ) {
			update_user_option( $user_id, 'ul_locked', $new_locked, false );
			if ( !$new_locked ) {
				update_user_option( $user_id, 'ul_bad_attempts', 0, false );
			}
		}
		
		$new_disabled = isset( $_POST['ul_disabled'] );
		update_user_option( $user_id, 'ul_disabled', $new_disabled, false );
	}
	
	// Add new column to the user list page
	function manage_users_columns( $columns ) {
		// This requires WP 2.8+
		global $wp_version;
		if ( version_compare( $wp_version, '2.8', '>=' ) ) {
			$columns['userlocker'] = __('Locked / Disabled', 'user-locker');
		}
		return $columns;
	}
	
	// Add column content for each user on user list
	function manage_users_custom_column( $value, $column_name, $user_id ) {
		if ( $column_name == 'userlocker' ) {
			if ( get_user_option( 'ul_locked', $user_id, false ) ) {
				$ret = '<b>'.__('Yes', 'user-locker').'</b>';
			} else {
				$ret = __('No', 'user-locker');
			}
			$ret .= ' / ';
			if ( get_user_option( 'ul_disabled', $user_id, false ) ) {
				$ret .= '<b>'.__('Yes', 'user-locker').'</b>';
			} else {
				$ret .= __('No', 'user-locker');
			}
			return $ret;
		}
		return $value;
	}
	
	function sanitize_nonnegative( $value ) {
		$value = (int)$value;
		if ( $value < 0 ) {
			$value = 0;
		}
		return $value;
	}
	
	// Display settings form
	function options_panel() {
?>
<div class="wrap">
<?php screen_icon(); ?>
<h2><?php _e('User Locker - Options', 'user-locker'); ?></h2>

<form name="dofollow" action="options.php" method="post">
<?php settings_fields( 'user-locker' ); ?>
<table class="form-table">

<tr><th colspan="2"><h3><?php _e('Account locking:', 'user-locker'); ?></h3></th></tr>

<tr>
<th scope="row" style="text-align:right; vertical-align:top;">
<label for="userlocker_max_attempts"><?php _e('Maximum invalid login attempts before account locking:', 'user-locker'); ?></label>
</th>
<td>
<input type="text" maxlength="6" size="10" id="userlocker_max_attempts" name="userlocker_max_attempts" value="<?php echo stripcslashes( get_option( 'userlocker_max_attempts' ) ); ?>" />
</td>
</tr>

</table>

<p class="submit">
<input type="submit" name="Submit" value="<?php _e('Save settings', 'user-locker'); ?>" /> 
</p>

</form>
</div>
<?php
	}
}

add_option( 'userlocker_max_attempts', 5 ); // Maximum invalid login attempts before account locking

$wp_user_locker = new UserLocker();

} // END

?>